jQuery.sap.require("sap.ui.model.json.JSONModel");
jQuery.sap.require("sas.stress.config.model.Hierarchy");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.config.model.HierarchyMaps");
jQuery.sap.require("sas.stress.config.model.HierarchyMapPair");
jQuery.sap.require("sas.stress.models.Model");
jQuery.sap.require("sas.stress.wkbench.widget.MessageDetailsBox");
jQuery.sap.require("sas.stress.wkbench.widget.NavChecker");

sap.ui.controller("sas.stress.config.view.HierarchyMapping",{

    onInit: function () {
		this.router = sap.ui.core.routing.Router.getRouter("router");  
		this.router.attachExactRouteMatched("HierarchyMapping",this._init,this);
    },
    _init : function(event) {
        var controller = this;
        var view = this.getView();
		controller.targetHierarchyKey=event.getParameter("arguments").hierarchyId;
		sas.stress.config.model.Hierarchy.getHierarchies().done(function(){
			var hierarchy = sas.stress.config.model.Hierarchy.getHierarchy(controller.targetHierarchyKey);

			var model=new sap.ui.model.json.JSONModel({
				hierarchyName: hierarchy.getName()
			});
			view.setModel(model);

	        var rb=sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
	        sap.ui.getCore().byId("contentInfoLabel").setText(rb.getText("hierarchyMappingPage.contentInfoLabel.fmt.txt", [hierarchy.getName()]));
	        sap.ui.getCore().byId("targetTreeTableLabel").setText(rb.getText("targetTree.title.fmt", [hierarchy.getName()]));

            controller._loadData();

	        sap.ui.getCore().byId("saveHierarchyMappingButton").setEnabled(false); //Init
		});
		var rb=sas.stress.wkbench.StressUtil.getBundle();
		this.navChecker=new sas.stress.wkbench.widget.NavChecker(sas.config.getAppId(), rb.getText("project.dirty.question.txt"), this.router);
        this.navChecker.start();
    },
    
    onExit: function(){
		this.router.detachExactRouteMatched("HierarchyMapping",this._init,this);
		this.navChecker.stop();
    },

    _loadData:function(){
        var deferred = $.Deferred();
        var controller = this;
        controller._getModels().done(function(){
            controller._loadHierarchyMappingInfo(controller.targetHierarchyKey).done(function(){
                deferred.resolve();
            }).fail(function(){
                deferred.reject();
            });
        }).fail(function(){
            deferred.reject();
        });
        return deferred;
    },

    _getModels:function(){
        var view = this.getView();
        var deferred = $.Deferred();
        sas.stress.models.Model.getModels().done(function(models){
            var modelsJson = [];
            _.each(models, function(model){
                modelsJson.push({
                    modelKey: model.getId(),
                    name: model.getName(),
                    description: model.getDescription(),
                    mappings: 0
                });
            });
            view.modelsTable.setModel(new sap.ui.model.json.JSONModel(modelsJson));
            view.modelsTable.bindRows("/");
            //Default sort by Name
            view.modelsTable.sort(view.modelsTable.getColumns()[0], sap.ui.table.SortOrder.Ascending);
            deferred.resolve();
        })
            .fail(function(){
                var rb=sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
                sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
                deferred.reject();
            });
        return deferred;
    },

    /* Gets info about all the source hierarchies that are mapped to the given target hierarchy.
    * Result also contains line items for the source hierarchies, line items for the target hierarchy
    * and line item mappings between the source hierarchies and target hierarchy.
    * This function then uses this data to set the view. */
    _loadHierarchyMappingInfo:function(targetHierarchyKey){
        var view = this.getView();
        var controller = this;
        var deferred = $.Deferred();
        sas.stress.config.model.HierarchyMaps.getHierarchyMaps(targetHierarchyKey).done(function(hierarchyMaps){
            controller.hierarchyMapPairs = hierarchyMaps.getHierarchyMapPairs(); //Mapping between hierarchies.
            //Build source and target hierarchy trees.
            var sourceHierarchiesRootNode = {children: []};
            controller.hierarchyMapInfos = hierarchyMaps.getHierarchyMapInfos(); //Cache this so it can be used in the Save function.
            _.each(hierarchyMaps.getHierarchyMapInfos(), function(hierarchyMapInfo){ //Contains line items for source and target hierarchies involved.
                if(hierarchyMapInfo.getHierarchyKey() != targetHierarchyKey){
                    //assume this is source hierarchy
                    //Add a node with this hierarchy name
                    var sourceHierarchy = sas.stress.config.model.Hierarchy.getHierarchy(hierarchyMapInfo.getHierarchyKey());
                    var sourceHierarchyRootNode = {name: sourceHierarchy.getName(), hierarchyKey: hierarchyMapInfo.getHierarchyKey(), children: []};
                    sourceHierarchiesRootNode.children.push(sourceHierarchyRootNode);
                    //Add nodes in the source hierarchy to the source hierarchy root node
                    _.each(hierarchyMapInfo.getLineItems(), function(lineItem){
                        controller._addToSourceTreeNode(sourceHierarchyRootNode, lineItem);
                    });
                } else {
                    var targetHierarchy = sas.stress.config.model.Hierarchy.getHierarchy(targetHierarchyKey);
                    var targetHierarchyRootNode = {name: targetHierarchy.getName(), hierarchyKey: hierarchyMapInfo.getHierarchyKey(), children: []};
                    _.each(hierarchyMapInfo.getLineItems(), function(lineItem){
                        controller._addToTargetTreeNode(targetHierarchyRootNode, lineItem);
                    });
                    var targetTreeTableModel = new sap.ui.model.json.JSONModel();
                    targetTreeTableModel.setData(targetHierarchyRootNode);
                    view.targetTreeTable.setModel(targetTreeTableModel);
                    view.targetTreeTable.bindRows("/children");
                }
            });
            var sourceTreeTableModel = new sap.ui.model.json.JSONModel();
            //Sort by Hierarchy name ascending
            sourceHierarchiesRootNode.children.sort(function(a,b){
                var aName = a.name.toLowerCase();
                var bName = b.name.toLowerCase();
                return ((aName < bName) ? -1 : ((aName > bName) ? 1 : 0));
            });
            sourceTreeTableModel.setData(sourceHierarchiesRootNode);
            view.sourceTreeTable.setModel(sourceTreeTableModel);
            view.sourceTreeTable.bindRows("/children");

            //Add mapped nodes to target hierarchy tree.
            //Add hierarchy nodes.
            _.each(hierarchyMaps.getHierarchyItemMaps(), function(hierarchyItemMap) { //For each source hierarchy to target hierarchy mapping
                _.each(hierarchyItemMap.getItemMapPairs(), function(itemMapPair) { //For each line item mapping in the source hierarchy to target hierarchy mapping
                    //find node in target tree with liKey = toLiKey.
                    var targetNode = controller._findNode(view.targetTreeTable.getModel().getData(), {"liKey": itemMapPair.getToLiKey()});
                    //find node in source tree for hierarchy matching the fromHierKey in the map pair and with liKey = fromLiKey.
                    //First, find the source tree.
                    var hierarchyMapPair = _.find(controller.hierarchyMapPairs, function (hierarchyMapPair) {
                        return hierarchyMapPair.getHierarchyMapKey() == hierarchyItemMap.getHierarchyMapKey();
                    });
                    if (hierarchyMapPair != undefined) {
                        var sourceNode = controller._findNode(view.sourceTreeTable.getModel().getData(),
                            {
                                "liKey": itemMapPair.getFromLiKey(),
                                "hierarchyKey": hierarchyMapPair.getFromHierKey()
                            });

                        //add a node under this node with liKey = fromLiKey
                        targetNode.children.push({
                            name: sourceNode.path,
                            liKey: itemMapPair.getFromLiKey(),
                            toLiKey: itemMapPair.getToLiKey(),
                            children: [],
                            hierarchyKey: sourceNode.hierarchyKey
                        });
                        //Update mappings count of the node in source tree.
                        sourceNode.mappings++;
                    }
                });
            });
            //Add model nodes.
            _.each(hierarchyMaps.getModelHierItemMaps(), function(modelHierItemMap) {
                //find node in target tree with liKey = toLiKey. modelHierItemMap.liKey is toLiKey.
                var targetNode = controller._findNode(view.targetTreeTable.getModel().getData(), {"liKey": modelHierItemMap.getLiKey()});
                //find model in models table.
                var modelData = _.find(view.modelsTable.getModel().getData(), function(data){
                    return data.modelKey == modelHierItemMap.getModelKey();
                });
                if(modelData != null) {
                    //add a node under the target node.
                    targetNode.children.push({
                        name: modelData.name,
                        modelKey: modelHierItemMap.getModelKey(),
                        toLiKey: modelHierItemMap.getLiKey(),
                        children: []
                    });
                    targetNode.supportsFormula = false; //Formula can only be given for node that does not have mapped model nodes.
                    //Update mappings for the model in the Source Models table.
                    modelData.mappings++;
                }
            });
            view.targetTreeTable.getModel().refresh();
            controller._toggleRemoveAllMappingsButton();
            deferred.resolve();
        })
            .fail(function(){
                var rb=sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
                sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
                deferred.reject();
            });
        return deferred;
    },

    /* Recursively searches the tree model for a node where node matches keyValueObj. Returns the node, if one is found. */
    _findNode : function(treeModelData, keyValueObj){
        var controller = this;
        for(var i=0; i < treeModelData.children.length; i++){
            var childNode = treeModelData.children[i];
            if(controller._isMatch(childNode, keyValueObj) === true) {
                return childNode;
            }
            //find in children of this node
            var node = controller._findNode(childNode, keyValueObj);
            if(node){
                return node;
            }
        }
    },

    //Copied from underscore.js as this function is not available in Underscore 1.6.
    // Returns whether an object has a given set of `key:value` pairs.
    _isMatch : function(object, attrs) {
        var keys = _.keys(attrs), length = keys.length;
        if (object == null) return !length;
        var obj = Object(object);
        for (var i = 0; i < length; i++) {
            var key = keys[i];
            if (attrs[key] !== obj[key] || !(key in obj)) return false;
        }
        return true;
    },


    /* Adds a branch of nodes listed in levelNames to the rootNode. */
    _addToSourceTreeNode : function(rootNode, lineItem){
        var currentNode = rootNode;
        $.each(lineItem.getLevelNames(), function(index, levelName){
            //check if currentNode.children contains node with levelName[i].
            var node = _.find(currentNode.children, function(childNode){
                return childNode.name === levelName;
            });
            //If not, add node with levelName[i].
            if(node == undefined){
                node = {name: levelName, hierarchyKey: lineItem.getHierarchyKey(), children: []};
                //set liKey and path on the leaf node.
                if(index == lineItem.getLevelNames().length - 1){
                    node.liKey = lineItem.getLiKey();
                    node.path = rootNode.name + "," + lineItem.getLevelNames();
                    node.mappings = 0;
                }
                currentNode.children.push(node);
            }
            //set currentNode as the node with levelName[i] for levelName[i+1].
            currentNode = node;
        });
    },

    /* Adds a branch of nodes listed in levelNames to the rootNode. */
    _addToTargetTreeNode : function(rootNode, lineItem){
        var currentNode = rootNode;
        $.each(lineItem.getLevelNames(), function(index, levelName){
            //check if currentNode.children contains node with levelName[i].
            var node = _.find(currentNode.children, function(childNode){
                return childNode.name === levelName;
            });
            //If not, add node with levelName[i].
            if(node == undefined){
                node = {name: levelName, hierarchyKey: lineItem.getHierarchyKey(), children: []};
                //set HierarchyLineItem properties on the last level node.
                if(index == lineItem.getLevelNames().length - 1){
                    node.liKey = lineItem.getLiKey();
                    node.liOrder = lineItem.getLiOrder();
                    node.contra = lineItem.getContra();
                    node.aggregation = lineItem.getAggregation();
                    node.format = lineItem.getFormat();
                    node.formula = lineItem.getFormula();
                    node.levelNames = lineItem.getLevelNames();
                    node.supportsFormula = true; //Init
                }
                currentNode.children.push(node);
            }
            //set currentNode as the node with levelName[i] for levelName[i+1].
            currentNode = node;
        });
    },

    sourceIconTabBarSelect:function(event){
        this._toggleAddMappingButton();
        this._toggleRemoveMappingButton();
        this._toggleRemoveAllMappingsButton();
    },

    addMappingButtonPress : function(event){
        var view = this.getView();
        var controller = this;
        var targetTreeTableSelectedIndices = view.targetTreeTable.getSelectedIndices();
        if(targetTreeTableSelectedIndices.length != 1){
            return; //return if no or more than 1 target node selected
        }
        //Get target node
        var selectedTargetNodePath = view.targetTreeTable.getContextByIndex(targetTreeTableSelectedIndices[0]).getPath();
        var selectedTargetNode = view.targetTreeTable.getModel().getProperty(selectedTargetNodePath);
        if(selectedTargetNode.formula != null && selectedTargetNode.formula.length > 0) {
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            sas.hc.m.MessageBox.warning(rb.getText("hierarchyMappingPage.removeFormula.confirmation.msg.txt"), rb.getText("hierarchyMappingPage.addMapping.window.title"),
                [sas.hc.m.MessageBox.Action.YES, sas.hc.m.MessageBox.Action.NO], null, function (action) {
                    if (action == sas.hc.m.MessageBox.Action.YES) {
                        selectedTargetNode.formula = undefined;
                        controller._addModelMapping();
                    }
                }
            );
        } else{
            controller._addModelMapping();
        }
    },

    _addModelMapping:function(){
        var view = this.getView();
        var controller = this;
        var targetTreeTableSelectedIndices = view.targetTreeTable.getSelectedIndices();
        var selectedTargetNodePath = view.targetTreeTable.getContextByIndex(targetTreeTableSelectedIndices[0]).getPath();
        var selectedTargetNode = view.targetTreeTable.getModel().getProperty(selectedTargetNodePath);
        view.targetTreeTable.expand(targetTreeTableSelectedIndices[0]); //expand the target node

        //Models tab selected
        var modelsTableSelectedIndices = view.modelsTable.getSelectedIndices();
        if(modelsTableSelectedIndices.length == 0){
            return; //return if no model selected
        }
        //Get the list of selected items in the models table.
        $.each(modelsTableSelectedIndices, function(index, selectedIndex){
            var context= view.modelsTable.getContextByIndex(selectedIndex);
            var selectedModel = context.getObject();
            //Check if model is not already mapped to this target node
            var model = _.find(selectedTargetNode.children, function(mappedNode){
                return mappedNode.modelKey != undefined && mappedNode.modelKey == selectedModel.modelKey;
            });
            if(model == undefined){
                //model not already mapped to this target node. Add selected model to target node.
                selectedTargetNode.children.push({
                    name: selectedModel.name,
                    modelKey: selectedModel.modelKey,
                    toLiKey: selectedTargetNode.liKey,
                    children: []
                });
                selectedTargetNode.supportsFormula = false; //Formula can only be given for node that does not have mapped model nodes.
                //Update source node to indicate it is mapped.
                selectedModel.mappings++;
                controller._setDirty(true);
            }
        });
        //Refresh models to update view.
        view.modelsTable.getModel().refresh();
        view.targetTreeTable.getModel().refresh();
        controller._toggleRemoveAllMappingsButton();
    },

    /* Recursively searches for all the leaf nodes under the given node. */
    _getLeafNodes : function(node){
        var controller = this;
        _.each(node.children, function(childNode){
            if(childNode.children.length == 0){
                controller.leafNodes.push(childNode);
            } else {
                controller._getLeafNodes(childNode);
            }
        });
        return controller.leafNodes;
    },

    removeMappingButtonPress : function(event){
        var view = this.getView();
        var controller = this;
        var targetTreeTableModel = view.targetTreeTable.getModel();

        //Get the list of selected nodes. We get this list first because getContextByIndex will return a different row once we remove a node.
        var selectedNodes = [];
        $.each(view.targetTreeTable.getSelectedIndices(), function(index, selectedIndex){
            var selectedNodePath = view.targetTreeTable.getContextByIndex(selectedIndex).getPath();
            var selectedNode = targetTreeTableModel.getProperty(selectedNodePath);
            selectedNodes.push(selectedNode);
        });

        controller._removeModelMappings(selectedNodes);
    },

    _removeModelMappings:function(selectedNodes){
        var view = this.getView();
        var controller = this;
        var targetTreeTableModel = view.targetTreeTable.getModel();
        $.each(selectedNodes, function(index, selectedNode){
            if(selectedNode.modelKey == undefined){ //only mapped model nodes can be removed
                return true;    //continue for loop
            }
            //Get parent node of the selected node.
            var parentNode = controller._findNode(targetTreeTableModel.getData(), {"liKey": selectedNode.toLiKey});
            //Remove selected node from parent node.
            $.each(parentNode.children, function (index, childNode) {
                if (childNode.modelKey == selectedNode.modelKey) {
                    parentNode.children.splice(index, 1);
                    return false; //break for loop
                }
            });
            //Update mappings property of the model in the Models source table.
            $.each(view.modelsTable.getModel().getData(), function (index, model) {
                if (model.modelKey == selectedNode.modelKey) {
                    model.mappings--;
                    return false; //break for loop
                }
            });
            parentNode.supportsFormula = !controller._containsMappedModelNodes(parentNode, false); //Formula can only be given for node that does not have mapped model nodes.
            controller._setDirty(true);
        });
        view.targetTreeTable.clearSelection();
        //Refresh the models for the views to update.
        targetTreeTableModel.refresh();
        view.modelsTable.getModel().refresh();
        controller._toggleRemoveAllMappingsButton();
    },

    removeAllMappingsButtonPress : function(event){
        var view = this.getView();
        this._removeAllModelMappings();
        view.targetTreeTable.clearSelection();
    },

    _removeAllModelMappings:function(){
        var view = this.getView();
        var controller = this;
        var targetTreeTableModel = view.targetTreeTable.getModel();
        //remove all mapped nodes from the target tree table.
        controller._removeAllMappedModelNodesFromTargetTree(targetTreeTableModel.getData());

        //Reset mappings property of models in the Models table.
        _.each(view.modelsTable.getModel().getData(), function(model){
            model.mappings = 0;
        });
        view.modelsTable.getModel().refresh();

        //Refresh the models for the views to update.
        targetTreeTableModel.refresh();

        controller._setDirty(true);
        sap.ui.getCore().byId("removeAllMappingsButton").setEnabled(false);
    },

    _removeAllMappedModelNodesFromTargetTree : function(modelData){
        var controller = this;
        _.each(modelData.children, function(node){
            if(controller._containsMappedModelNodes(node, false)){
                //Remove all child model nodes.
                for(var i=node.children.length - 1; i >= 0; i--) {
                    if(node.children[i].modelKey != undefined){
                        node.children.splice(i, 1);
                    }
                }
                node.supportsFormula = true; //Formula can only be given for node that does not have mapped model nodes.
            } else { //remove mapped nodes from children of this node
                controller._removeAllMappedModelNodesFromTargetTree(node);
            }
        });
    },

    saveHierarchyMappingButtonPress : function(event){
        var view = this.getView();
        var controller = this;
        this._saveHierarchyMappingPage().done(function(){
            controller._setDirty(false);
            var rb=sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            sap.m.MessageToast.show(rb.getText("hierarchyMappingPage.save.successful.msg.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
            //Reload data in the view
            controller._loadData().done(function(){
                //Reset Expand/Collapse button state
                var sourceTreeTableExpandCollapseButton = sap.ui.getCore().byId("sourceTreeTableExpandCollapseButton");
                sourceTreeTableExpandCollapseButton.setIcon(sas.icons.HC.EXPANDSECTION);
                sourceTreeTableExpandCollapseButton.setTooltip(rb.getText("sourceTreeTable.expandCollapseButton.tip"));
                view.sourceTreeTableExpanded=false;
                var targetTreeTableExpandCollapseButton = sap.ui.getCore().byId("targetTreeTableExpandCollapseButton");
                targetTreeTableExpandCollapseButton.setIcon(sas.icons.HC.EXPANDSECTION);
                targetTreeTableExpandCollapseButton.setTooltip(rb.getText("sourceTreeTable.expandCollapseButton.tip"));
                view.targetTreeTableExpanded=false;
            });
        });
    },

    _saveHierarchyMappingPage : function(){
        var deferred = $.Deferred();
        var view = this.getView();
        var controller = this;
        //Get mapped nodes (hierarchy and model).
        controller.mappedData = {hierarchyItemMapPairs: [], modelItemMaps: []}; //Initialize, we'll use this var to store mapped nodes.
        var mappedData = controller._getMappedNodes(view.targetTreeTable.getModel().getData());
        delete controller.mappedData; //Done using this var.
        var hierarchyItemMaps = [];
        _.each(controller.hierarchyMapPairs, function(hierarchyMapPair){
            //Get line item mappings for hierarchyMapPair.
            var hierarchyItemMapPairs = _.filter(mappedData.hierarchyItemMapPairs, function(hierarchyItemMapPair){
                return hierarchyItemMapPair.hierarchyKey == hierarchyMapPair.getFromHierKey();
            });
            var itemMapPairs = [];
            _.each(hierarchyItemMapPairs, function(hierarchyItemMapPair){
                itemMapPairs.push(new sas.stress.config.model.HierarchyItemMapDetail(hierarchyMapPair.getHierarchyMapKey(),
                    hierarchyItemMapPair.liKey, hierarchyItemMapPair.toLiKey));
            });
            hierarchyItemMaps.push(new sas.stress.config.model.HierarchyItemMap(hierarchyMapPair.getHierarchyMapKey(), itemMapPairs));
        });

        var hierarchyMapInfos = controller.hierarchyMapInfos;
        _.each(hierarchyMapInfos, function(hierarchyMapInfo){
            if(hierarchyMapInfo.getHierarchyKey() == controller.targetHierarchyKey){
                //Since there can be formula changes, we need to build list of line items from the target tree.
                var lineItems = [];
                controller._getLineItemsInTargetTree(view.targetTreeTable.getModel().getData(), lineItems);
                hierarchyMapInfo.setLineItems(lineItems);
            }
        });

        var modelHierItemMaps = [];
        _.each(mappedData.modelItemMaps, function(modelItemMap){
            modelHierItemMaps.push(new sas.stress.config.model.ModelHierItemMap(modelItemMap.modelKey, controller.targetHierarchyKey, modelItemMap.toLiKey));
        });
        var hierarchyMaps = new sas.stress.config.model.HierarchyMaps([], hierarchyItemMaps, hierarchyMapInfos, modelHierItemMaps);
        sas.stress.config.StressAPI.updateTargetMappings(controller.targetHierarchyKey, hierarchyMaps).done(function(){
            deferred.resolve();
        })
            .fail(function(jqxhr){
                sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
            });
        return deferred;
    },

    /* Gets nodes in the target tree having liKey and matching target hierarchy key and puts them in the lineItems array. */
    _getLineItemsInTargetTree:function(node, lineItems){
        var controller = this;
        _.each(node.children, function(childNode){
            if(childNode.liKey != null && childNode.hierarchyKey == controller.targetHierarchyKey){
                var hierarchyLineItem = new sas.stress.config.model.HierarchyLineItem(childNode.liKey, childNode.hierarchyKey,
                    childNode.liOrder, childNode.contra, childNode.aggregation, childNode.levelNames, childNode.format, childNode.formula);
                lineItems.push(hierarchyLineItem);
            }
            if(childNode.children.length > 0){
                controller._getLineItemsInTargetTree(childNode, lineItems);
            }
        });
    },

    /* Recursively searches for all mapped nodes under the given node. */
    _getMappedNodes : function(node){
        var controller = this;
        _.each(node.children, function(childNode){
            if(childNode.children.length == 0){
                if(childNode.hierarchyKey != undefined && childNode.hierarchyKey != controller.targetHierarchyKey){ //Hierarchy node
                    controller.mappedData.hierarchyItemMapPairs.push(childNode);
                } else if(childNode.modelKey != undefined) { //Model node
                    controller.mappedData.modelItemMaps.push(childNode);
                }
            } else {
                controller._getMappedNodes(childNode);
            }
        });
        return controller.mappedData;
    },

    /* Searches for mapped model nodes, under the given node and returns true if it finds at least 1 mapped model node. Returns false if no mapped model nodes found.
    * Searches recursively if recursive param is set to true. */
    _containsMappedModelNodes : function(node, recursive){
        var controller = this;
        var mappedNodeExists = false;
        $.each(node.children, function(index, childNode){
            if(childNode.children.length == 0){
                mappedNodeExists = (childNode.modelKey != undefined);
            } else if(recursive === true) {
                mappedNodeExists = controller._containsMappedModelNodes(childNode, true);
            }
            if(mappedNodeExists === true){
                return false; //break for loop
            }
        });
        return mappedNodeExists;
    },

    breadcrumbClickHandler : function(event) {
        var controller = this;
        switch(event.getParameter("item").getKey()) {
            case "Configure":
            	sas.stress.wkbench.ViewManager.loadProjectConfigurationEditor();
                /*this._readyToLeaveView().done(function(leaveView){
                    if(leaveView === true) {
                        sas.stress.wkbench.ViewManager.loadProjectConfigurationEditor();
                    }
                });*/
                break;
            case "Home":
            	sas.stress.wkbench.ViewManager.loadProjectManager();
                /*this._readyToLeaveView().done(function(leaveView){
                    if(leaveView === true) {
                        sas.stress.wkbench.ViewManager.loadProjectManager();
                    }
                });*/
                break;
        }
    },

    /* Called when the user selects breadcrumb of a previous view. Checks if current view is dirty and if so prompts to Save.
    *  Returns true if view can be changed, false otherwise. */
    _readyToLeaveView:function(){
        var deferred = $.Deferred();
        var controller = this;
        if(sap.ui.getCore().byId("saveHierarchyMappingButton").getEnabled() == true){ //Prompt to Save if dirty
            var rb=sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            sas.hc.m.MessageBox.warning(rb.getText("hierarchyMappingPage.save.confirmation.msg.txt"), sas.config.appName,
                [sas.hc.m.MessageBox.Action.SAVE, sas.hc.m.MessageBox.Action.DONT_SAVE, sas.hc.m.MessageBox.Action.CANCEL], null, function(oAction){
                    if(oAction == sas.hc.m.MessageBox.Action.SAVE){
                        controller._saveHierarchyMappingPage().done(function(){
                            controller._setDirty(false);
                            deferred.resolve(true);
                        });
                    } else if(oAction == sas.hc.m.MessageBox.Action.DONT_SAVE){
                        deferred.resolve(true);
                    } else {
                        deferred.resolve(false);
                    }
                }
            );
        } else {
            deferred.resolve(true);
        }
        return deferred;
    },

    sourceTreeTableAddButtonPress:function(event){
        var view = this.getView();
        var controller = this;
        var existingDialog = sap.ui.getCore().byId("sas.stress.config.view.AddHierarchyMappingDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var dialog = sap.ui.jsfragment("sas.stress.config.view.AddHierarchyMappingDialog", this);
        var sourceHierarchies = _.filter(sas.stress.config.model.Hierarchy.getCachedHierarchies(), function(hierarchy){
            var hierarchyAlreadyAdded = _.find(view.sourceTreeTable.getModel().getData().children, function(sourceHierarchy){
                return sourceHierarchy.hierarchyKey == hierarchy.getHierarchyKey();
            });
            return hierarchy.getHierarchyKey() != controller.targetHierarchyKey && hierarchyAlreadyAdded == undefined;
        });
        var model=new sap.ui.model.json.JSONModel({sourceHierarchies: sourceHierarchies});
        dialog.setModel(model);
        dialog.open();
        // workaround for sort because commons messes up the table header alignment.. defect logged S1179681
        setTimeout(function(){
            var sourceHierarchiesTable = sap.ui.getCore().byId("AddHierarchyMappingDialog.sourceHierarchiesTable");
            sourceHierarchiesTable.sort(sourceHierarchiesTable.getColumns()[0], sap.ui.table.SortOrder.Ascending);
        },200);
    },

    addHierarchyMappingDialogSearch:function(event){
        var value=event.getParameter("value")||event.getParameter("newValue")|| event.getParameter("query") || "";
        var filter1=new sap.ui.model.Filter("name",sap.ui.model.FilterOperator.Contains,value);
        var filter2=new sap.ui.model.Filter("description",sap.ui.model.FilterOperator.Contains,value);
        var filter3=new sap.ui.model.Filter("categoryName",sap.ui.model.FilterOperator.Contains,value);
        var allFilters=new sap.ui.model.Filter([filter1,filter2, filter3]);
        var sourceHierarchiesTable = sap.ui.getCore().byId("AddHierarchyMappingDialog.sourceHierarchiesTable");
        var binding=sourceHierarchiesTable.getBinding("rows");
        binding.filter(allFilters);
    },

    sourceHierarchiesTableRowSelectionChange:function(event){
        var sourceHierarchiesTable = sap.ui.getCore().byId("AddHierarchyMappingDialog.sourceHierarchiesTable");
        var saveButton = sap.ui.getCore().byId("AddHierarchyMappingDialog.saveButton");
        saveButton.setEnabled(sourceHierarchiesTable.getSelectedIndices().length > 0);
    },

    addHierarchyMappingDialogSaveButtonPress: function(event) {
        var view = this.getView();
        var controller = this;
        var dialog = sap.ui.getCore().byId("sas.stress.config.view.AddHierarchyMappingDialog");
        dialog.setBusy(true);
        var sourceHierarchiesTable = sap.ui.getCore().byId("AddHierarchyMappingDialog.sourceHierarchiesTable");
        var hierarchyMapPairs = [];
        _.each(sourceHierarchiesTable.getSelectedIndices(), function(selectedIndex){
            var context= sourceHierarchiesTable.getContextByIndex(selectedIndex);
            var obj=context.getObject();
            hierarchyMapPairs.push(new sas.stress.config.model.HierarchyMapPair(-1, parseInt(obj.hierarchyKey),
                parseInt(controller.targetHierarchyKey), ""));
        });
        if(hierarchyMapPairs.length > 0){
            sas.stress.config.model.HierarchyMapPair.addHierarchyMapPairs(hierarchyMapPairs).done(function(){
                dialog.setBusy(false);
                dialog.close();
                view.sourceTreeTable.clearSelection();
                //Reload data in the view
                controller._loadData();
            })
                .fail(function(jqxhr){
                    dialog.setBusy(false);
                    sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
                });
        } else {
            dialog.setBusy(false);
            dialog.close();
        }
    },

    sourceTreeTableRemoveButtonPress:function(event){
        var view = this.getView();
        var controller = this;
        var hierarchyMapKeys = [];
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var removeMessage = "";
        _.each(view.sourceTreeTable.getSelectedIndices(), function(selectedIndex){
            var selectedNodePath = view.sourceTreeTable.getContextByIndex(selectedIndex).getPath();
            var selectedNode = view.sourceTreeTable.getModel().getProperty(selectedNodePath);
            //Check if selected node is the root node of the hierarchy. Ignore non-root nodes.
            var hierarchyRootNode = _.find(view.sourceTreeTable.getModel().getData().children, function (data) {
                return selectedNode.name === data.name;
            });
            if(hierarchyRootNode != undefined) {
                var hierarchyMapPair = _.find(controller.hierarchyMapPairs, function(hierarchyMapPair){
                    return hierarchyMapPair.getFromHierKey() == hierarchyRootNode.hierarchyKey;
                });
                if(hierarchyMapPair != null) {
                    hierarchyMapKeys.push(hierarchyMapPair.getHierarchyMapKey());
                    if(view.sourceTreeTable.getSelectedIndices().length == 1) {
                        removeMessage = rb.getText("hierarchyMapping.removeHierarchy.single.confirmation.msg.fmt.txt", [hierarchyRootNode.name]);
                    }
                }
            }
        });
        if(hierarchyMapKeys.length > 0) {
            if(hierarchyMapKeys.length > 1){
                removeMessage = rb.getText("hierarchyMapping.removeHierarchy.multiple.confirmation.msg.fmt.txt", [hierarchyMapKeys.length]);
            }
            sas.hc.m.MessageBox.warning(removeMessage, rb.getText("hierarchyMapping.removeHierarchy.window.title"),
                [sas.hc.m.MessageBox.Action.DELETE, sas.hc.m.MessageBox.Action.CANCEL], null, function (oAction) {
                    if (oAction == sas.hc.m.MessageBox.Action.DELETE) {
                        sas.stress.config.model.HierarchyMapPair.deleteHierarchyMapPairs(hierarchyMapKeys)
                            .done(function () {
                                view.sourceTreeTable.clearSelection();
                                view.targetTreeTable.clearSelection();
                                //Reload data in the view
                                controller._loadData();
                            })
                            .fail(function (jqxhr) {
                                sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
                            });
                    }
                }
            );
        }
    },

    sourceTreeTableExpandButtonPress: function(event) {
        var view = this.getView();
        for(var i=0; i<view.sourceTreeTable.getBinding("rows").getLength(); i++){
            view.sourceTreeTable.expand(i);
        }
    },

    sourceTreeTableCollapseButtonPress: function(event){
        var view = this.getView();
        for(var i=view.sourceTreeTable.getBinding("rows").getLength()-1; i>=0; i--){
            view.sourceTreeTable.collapse(i);
        }
    },

    targetTreeTableExpandButtonPress: function(event) {
        var view = this.getView();
        for(var i=0; i<view.targetTreeTable.getBinding("rows").getLength(); i++){
            view.targetTreeTable.expand(i);
        }
    },

    targetTreeTableCollapseButtonPress: function(event){
        var view = this.getView();
        for(var i=view.targetTreeTable.getBinding("rows").getLength()-1; i>=0; i--){
            view.targetTreeTable.collapse(i);
        }
    },

    sourceTreeTableRowSelectionChange:function(event){
        this._toggleRemoveHierarchyButton();
    },

    /* Enables Remove hierarchy button only if view is not dirty and all selected nodes are hierarchy root nodes.*/
    _toggleRemoveHierarchyButton:function(){
        var view = this.getView();
        var removeHierarchyButton = sap.ui.getCore().byId("sourceTreeTableRemoveButton");
        if(sap.ui.getCore().byId("saveHierarchyMappingButton").getEnabled() === false) {
            var hierarchyNodesSelected = 0;
            var selectedIndices = view.sourceTreeTable.getSelectedIndices();
            $.each(selectedIndices, function (index, selectedIndex) {
                var selectedNodePath = view.sourceTreeTable.getContextByIndex(selectedIndex).getPath();
                var selectedNode = view.sourceTreeTable.getModel().getProperty(selectedNodePath);
                //Check if selected node is the root node of the hierarchy.
                var hierarchyRootNode = _.find(view.sourceTreeTable.getModel().getData().children, function (data) {
                    return selectedNode.name === data.name;
                });
                if (hierarchyRootNode != undefined) {
                    hierarchyNodesSelected++;
                } else{
                    return false; //break for loop
                }
            });
            removeHierarchyButton.setEnabled(hierarchyNodesSelected > 0 && hierarchyNodesSelected == selectedIndices.length);
        } else{
            removeHierarchyButton.setEnabled(false);
        }
    },

    targetTreeTableFormulaLiveChange:function(event){
        this._setDirty(true);
    },

    targetTreeTableRowSelectionChange:function(event){
        this._toggleAddMappingButton();
        this._toggleRemoveMappingButton();
    },

    modelsTableRowSelectionChange:function(event){
        this._toggleAddMappingButton();
    },

    _toggleRemoveAllMappingsButton:function(){
        var view = this.getView();
        var controller = this;
        var removeAllMappingsButton = sap.ui.getCore().byId("removeAllMappingsButton");
        //Enable Remove All button if there are model mappings to remove.
        removeAllMappingsButton.setEnabled(controller._containsMappedModelNodes(view.targetTreeTable.getModel().getData(), true));
    },

    _toggleRemoveMappingButton:function(){
        var view = this.getView();
        var removeMappingButton = sap.ui.getCore().byId("removeMappingButton");
        //Enable Remove button if at least 1 mapped model node is selected.
        var node = _.find(view.targetTreeTable.getSelectedIndices(), function (selectedIndex) {
            var selectedNodePath = view.targetTreeTable.getContextByIndex(selectedIndex).getPath();
            var selectedNode = view.targetTreeTable.getModel().getProperty(selectedNodePath);
            return selectedNode.modelKey != undefined;
        });
        removeMappingButton.setEnabled((node != undefined));
    },

    _toggleAddMappingButton:function(){
        var view = this.getView();
        var controller = this;
        var addMappingButton = sap.ui.getCore().byId("addMappingButton");
        var targetTreeTableSelectedIndices = view.targetTreeTable.getSelectedIndices();
        if(targetTreeTableSelectedIndices.length != 1){
            addMappingButton.setEnabled(false);
            return; //return if no or more than 1 target node selected
        }

        //Find which source tab user is on.
        var sourceTabs = sap.ui.getCore().byId("HierarchyMapping.sourceTabs");
        if(sourceTabs.getSelectedKey() == "sourceHierarchiesTab"){
            addMappingButton.setEnabled(false);
            return; //return if Hierarchies tab selected
        } else { //Models tab selected
            if(view.modelsTable.getSelectedIndices().length == 0){
                addMappingButton.setEnabled(false);
                return; //return if no model selected
            }
        }

        //Get target node
        var targetTreeTableModel = view.targetTreeTable.getModel();
        var selectedTargetNodePath = view.targetTreeTable.getContextByIndex(targetTreeTableSelectedIndices[0]).getPath();
        var selectedTargetNode = targetTreeTableModel.getProperty(selectedTargetNodePath);
        //map only to node in the target tree that is not a mapped node and is a leaf node in the target tree
        // (i.e. node that contains only mapped nodes, if any).
        var isLeafNodeInTargetTree = (selectedTargetNode.hierarchyKey != undefined && selectedTargetNode.hierarchyKey == controller.targetHierarchyKey
            && (selectedTargetNode.children.length == 0 || selectedTargetNode.children[0].modelKey != undefined
            || selectedTargetNode.children[0].hierarchyKey != controller.targetHierarchyKey));
        addMappingButton.setEnabled(isLeafNodeInTargetTree);
    },

    _setDirty:function(isDirty){
        sap.ui.getCore().byId("saveHierarchyMappingButton").setEnabled(isDirty); //Enable Save button
        sap.ui.getCore().byId("sourceTreeTableAddButton").setEnabled(!isDirty); //Disable Add button if dirty
        this._toggleRemoveHierarchyButton(); //Disable Remove button if dirty
        // Let Appswitcher know
        this.navChecker.setDirty(isDirty);
        //sas.applicationSwitcher.updateDirtyFlag(sas.config.getAppId(), isDirty);
    },

    targetTreeTableMaxButtonPress:function(event){
        var view=this.getView();
        var maxButton = sap.ui.getCore().byId("targetTreeTableMaxButton");
        if (view.targetTreeTableMaximized == false){
            maxButton.setIcon(sas.icons.HC.NORMALVIEW);
            view.splitterPosition=view.splitter.getSplitterPosition();
            view.splitter.setSplitterPosition("0%");
            view.targetTreeTableMaximized=true;
        } else{
            maxButton.setIcon(sas.icons.HC.MAXIMIZEVIEW);
            view.splitter.setSplitterPosition(view.splitterPosition);
            view.targetTreeTableMaximized=false;
        }
    }
});
